# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2021-2022 mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
API docs for mapsets
"""

from actinia_core.models.response_models import (
    LockedMapsetListResponseModel,
    SimpleResponseModel,
)

__license__ = "GPLv3"
__author__ = "Julia Haas, Guido Riembauer, Anika Weinmann"
__copyright__ = "Copyright 2021-2022, mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"

get_doc = {
    "tags": ["Mapsets"],
    "description": "List available or locked mapsets.",
    "parameters": [
        {
            "in": "path",
            "name": "mapsets",
            "type": "string",
            "description": "List all mapsets in the global database available "
            "to the authenticated user.",
        },
        {
            "in": "path",
            "name": "status",
            "type": "string",
            "description": (
                "If set to 'locked', list all locked mapsets across "
                "all locations. Minimum required user role: admin."
            ),
        },
        {
            "in": "path",
            "name": "user",
            "type": "string",
            "description": (
                "List all mapsets in the global database available "
                "to the specified user. "
                "Minimum required user role: admin"
            ),
        },
    ],
    "responses": {
        "200": {
            "description": "Returns a list of available (or locked) mapsets ",
            "schema": LockedMapsetListResponseModel,
        },
        "500": {
            "description": "The error message and a detailed error log",
            "schema": SimpleResponseModel,
        },
    },
}



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Model for Mapset management resources
"""
from copy import deepcopy
from actinia_core.models.response_models import ProcessingResponseModel

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Carmen Tawalika"
__copyright__ = "Copyright 2016-2021, Sören Gebbert and mundialis GmbH & Co. KG"
__maintainer__ = "mundialis"


class MapsetLockManagementResponseModel(ProcessingResponseModel):
    """The response content that is returned by the GET request
    """
    type = 'object'
    properties = deepcopy(ProcessingResponseModel.properties)
    properties["process_results"] = {}
    properties["process_results"]["type"] = "boolean"
    required = deepcopy(ProcessingResponseModel.required)
    example = {
        "accept_datetime": "2018-05-02 11:03:26.529673",
        "accept_timestamp": 1525259006.5296717,
        "api_info": {
            "endpoint": "mapsetlockmanagementresource",
            "method": "GET",
            "path": "/locations/nc_spm_08/mapsets/PERMANENT/lock",
            "request_url": "http://localhost:8080/locations/nc_spm_08/mapsets/"
                           "PERMANENT/lock"
        },
        "datetime": "2018-05-02 11:03:26.586348",
        "http_code": 200,
        "message": "Mapset lock state: False",
        "process_chain_list": [],
        "process_log": [],
        "process_results": False,
        "progress": {
            "num_of_steps": 0,
            "step": 0
        },
        "resource_id": "resource_id-162101d9-2abc-417e-83ef-dc6f52ed7aaf",
        "status": "finished",
        "time_delta": 0.056743621826171875,
        "timestamp": 1525259006.5863316,
        "urls": {
            "resources": [],
            "status": "http://localhost:8080/resources/admin/"
                      "resource_id-162101d9-2abc-417e-83ef-dc6f52ed7aaf"
        },
        "user_id": "admin"
    }



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
API docs for strds_raster_management
"""

from actinia_core.models.response_models import (
    ProcessingErrorResponseModel,
    ProcessingResponseModel,
)

from actinia_api.swagger2.actinia_core.schemas.strds_raster_management import (
    STRDSRasterListResponseModel,
    RasterListRegisterModel,
)

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Carmen Tawalika, Anika Weinmann"
__copyright__ = "Copyright 2016-2022, mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"

get_doc = {
    "tags": ["STRDS Management"],
    "description": "Get a list of all raster map layers that are registered "
    "in a STRDS that is located in a specific location/mapset. "
    "Minimum required user role: user.",
    "parameters": [
        {
            "name": "location_name",
            "description": "The name of the location",
            "required": True,
            "in": "path",
            "type": "string",
        },
        {
            "name": "mapset_name",
            "description": "The name of the mapset",
            "required": True,
            "in": "path",
            "type": "string",
        },
        {
            "name": "strds_name",
            "description": "The name of the STRDS",
            "required": True,
            "in": "path",
            "type": "string",
        },
        {
            "name": "where",
            "description": "A where statement to select user specific "
            "raster map layers from the STRDS",
            "required": False,
            "in": "query",
            "type": "string",
        },
    ],
    "responses": {
        "200": {
            "description": "This response returns a list of all raster map "
            "layers that are registered a specific STRDS and "
            "the log of the process chain that was used to "
            "create the response.",
            "schema": STRDSRasterListResponseModel,
        },
        "400": {
            "description": "The error message and a detailed log why "
            "creating a list of raster map layers from STRDS "
            "did not succeeded",
            "schema": ProcessingErrorResponseModel,
        },
    },
}

put_doc = {
    "tags": ["STRDS Management"],
    "description": "Register raster map layers in a STRDS located in a "
    "specific location/mapset. "
    "Minimum required user role: user.",
    "parameters": [
        {
            "name": "location_name",
            "description": "The name of the location",
            "required": True,
            "in": "path",
            "type": "string",
        },
        {
            "name": "mapset_name",
            "description": "The name of the mapset",
            "required": True,
            "in": "path",
            "type": "string",
        },
        {
            "name": "strds_name",
            "description": "The name of the STRDS",
            "required": True,
            "in": "path",
            "type": "string",
        },
        {
            "name": "raster_list",
            "description": "The list of raster map layers to be registered "
            "in the STRDS",
            "required": True,
            "in": "body",
            "schema": RasterListRegisterModel,
        },
    ],
    "responses": {
        "200": {
            "description": "Registration of raster map layers was "
            "successfully finished.",
            "schema": ProcessingResponseModel,
        },
        "400": {
            "description": "The error message and a detailed log why raster "
            "map layer registration did not succeeded",
            "schema": ProcessingErrorResponseModel,
        },
    },
}

delete_doc = {
    "tags": ["STRDS Management"],
    "description": "Unregister raster map layers from a STRDS located in a "
    "specific location/mapset. "
    "Minimum required user role: user.",
    "parameters": [
        {
            "name": "location_name",
            "description": "The name of the location",
            "required": True,
            "in": "path",
            "type": "string",
        },
        {
            "name": "mapset_name",
            "description": "The name of the mapset",
            "required": True,
            "in": "path",
            "type": "string",
        },
        {
            "name": "strds_name",
            "description": "The name of the STRDS",
            "required": True,
            "in": "path",
            "type": "string",
        },
        {
            "name": "raster_list",
            "description": "The list of raster map layers to be "
            "unregistered from the STRDS",
            "required": True,
            "in": "body",
            "schema": {"type": "array", "items": {"type": "string"}},
        },
    ],
    "responses": {
        "200": {
            "description": "Unregistration of raster map layers was "
            "successfully finished.",
            "schema": ProcessingResponseModel,
        },
        "400": {
            "description": "The error message and a detailed log why raster "
            "map layer unregistration did not succeeded",
            "schema": ProcessingErrorResponseModel,
        },
    },
}



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
API docs for raster_layer
"""

from actinia_core.models.response_models import (
    ProcessingResponseModel,
    ProcessingErrorResponseModel,
)

from actinia_api import URL_PREFIX
from actinia_api.swagger2.actinia_core.schemas.raster_layer import (
    RasterInfoResponseModel,
)

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Carmen Tawalika, Guido Riembauer, Anika Weinmann"
__copyright__ = "Copyright 2016-2022, mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"

get_doc = {
    "tags": ["Raster Management"],
    "description": "Get information about an existing raster map layer. "
    "Minimum required user role: user.",
    "parameters": [
        {
            "name": "location_name",
            "description": "The location name",
            "required": True,
            "in": "path",
            "type": "string",
            "default": "nc_spm_08",
        },
        {
            "name": "mapset_name",
            "description": "The name of the mapset that contains the required "
            "raster map layer",
            "required": True,
            "in": "path",
            "type": "string",
            "default": "PERMANENT",
        },
        {
            "name": "raster_name",
            "description": "The name of the raster map layer to get "
            "information about",
            "required": True,
            "in": "path",
            "type": "string",
            "default": "elevation",
        },
    ],
    "consumes": ["application/json"],
    "produces": ["application/json"],
    "responses": {
        "200": {
            "description": "The raster map layer information",
            "schema": RasterInfoResponseModel,
        },
        "400": {
            "description": "The error message and a detailed log why "
            "gathering raster map "
            "layer information did not succeeded",
            "schema": ProcessingErrorResponseModel,
        },
    },
}

delete_doc = {
    "tags": ["Raster Management"],
    "description": "Delete an existing raster map layer. Minimum required "
    "user role: user.",
    "parameters": [
        {
            "name": "location_name",
            "description": "The location name",
            "required": True,
            "in": "path",
            "type": "string",
        },
        {
            "name": "mapset_name",
            "description": "The name of the mapset that contains the "
            "required raster map layer",
            "required": True,
            "in": "path",
            "type": "string",
        },
        {
            "name": "raster_name",
            "description": "The name of the raster map layer to be deleted",
            "required": True,
            "in": "path",
            "type": "string",
        },
    ],
    "produces": ["application/json"],
    "responses": {
        "200": {
            "description": "Successfuly delete a raster map layer",
            "schema": ProcessingResponseModel,
        },
        "400": {
            "description": "The error message and a detailed log why "
            "raster map "
            "layer deletion did not succeeded",
            "schema": ProcessingErrorResponseModel,
        },
    },
}

post_doc = {
    "tags": ["Raster Management"],
    "description": "Create a new raster map layer by uploading a GeoTIFF. "
    "This method will fail if the map already exists. "
    'An example request is \'curl -L -u "XXX:XXX" -X POST '
    '-H "Content-Type: multipart/form-data" -F '
    '"file=@/home/....tif" http://localhost:8088'
    f"{URL_PREFIX}/"
    "locations/nc_spm_08/mapsets/test_mapset/raster_layers/"
    "testraster'. Minimum required user role: user.",
    "parameters": [
        {
            "name": "location_name",
            "description": "The location name",
            "required": True,
            "in": "path",
            "type": "string",
        },
        {
            "name": "mapset_name",
            "description": "The name of the mapset in which the raster map "
            "layer should be created",
            "required": True,
            "in": "path",
            "type": "string",
        },
        {
            "name": "raster_name",
            "description": "The name of the new raster map layer to be "
            "created",
            "required": True,
            "in": "path",
            "type": "string",
        },
    ],
    "consumes": ["Content-Type: multipart/form-data"],
    "produces": ["application/json"],
    "responses": {
        "200": {
            "description": "Raster map layer import information",
            "schema": ProcessingResponseModel,
        },
        "400": {
            "description": "The error message and a detailed log why raster "
            "map layer import failed",
            "schema": ProcessingErrorResponseModel,
        },
    },
}



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2019 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
STRDS map layer management

TODO: Integrate into the ephemeral process chain approach
"""
from copy import deepcopy
from flask_restful_swagger_2 import Schema
from actinia_core.models.response_models import ProcessingResponseModel

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Carmen Tawalika"
__copyright__ = "Copyright 2016-2021, Sören Gebbert and mundialis GmbH & Co. KG"
__maintainer__ = "mundialis"


class STRDSInfoModel(Schema):
    """Schema that contains space-time raster dataset (STRDS) information
    """
    description = "Information about a specific space-time raster dataset (STRDS)"
    type = 'object'
    properties = {
        "aggregation_type": {'type': 'string'},
        "bottom": {'type': 'string'},
        "creation_time": {'type': 'string'},
        "creator": {'type': 'string'},
        "east": {'type': 'string'},
        "end_time": {'type': 'string'},
        "ewres_max": {'type': 'string'},
        "ewres_min": {'type': 'string'},
        "granularity": {'type': 'string'},
        "id": {'type': 'string'},
        "map_time": {'type': 'string'},
        "mapset": {'type': 'string'},
        "max_max": {'type': 'string'},
        "max_min": {'type': 'string'},
        "min_max": {'type': 'string'},
        "min_min": {'type': 'string'},
        "modification_time": {'type': 'string'},
        "name": {'type': 'string'},
        "north": {'type': 'string'},
        "number_of_semantic_labels": {'type': 'string'},
        "nsres_max": {'type': 'string'},
        "nsres_min": {'type': 'string'},
        "number_of_maps": {'type': 'string'},
        "raster_register": {'type': 'string'},
        "semantic_labels": {'type': 'string'},
        "semantic_type": {'type': 'string'},
        "south": {'type': 'string'},
        "start_time": {'type': 'string'},
        "temporal_type": {'type': 'string'},
        "top": {'type': 'string'},
        "west": {'type': 'string'}
    }
    example = {
        "aggregation_type": "None",
        "semantic_labels": "None",
        "bottom": "0.0",
        "creation_time": "2016-08-11 16:44:29.756411",
        "creator": "soeren",
        "east": "75.5",
        "end_time": "2013-07-01 00:00:00",
        "ewres_max": "0.25",
        "ewres_min": "0.25",
        "granularity": "1 month",
        "id": "precipitation_1950_2013_monthly_mm@PERMANENT",
        "map_time": "interval",
        "mapset": "PERMANENT",
        "max_max": "1076.9",
        "max_min": "168.9",
        "min_max": "3.2",
        "min_min": "0.0",
        "modification_time": "2016-08-11 16:45:14.032432",
        "name": "precipitation_1950_2013_monthly_mm",
        "north": "75.5",
        "nsres_max": "0.25",
        "nsres_min": "0.25",
        "number_of_semantic_labels": "None",
        "number_of_maps": "762",
        "raster_register": "raster_map_register_934719ed2b4841818386a6f9c5f11b09",
        "semantic_type": "mean",
        "south": "25.25",
        "start_time": "1950-01-01 00:00:00",
        "temporal_type": "absolute",
        "top": "0.0",
        "west": "-40.5"
    }


class STRDSInfoResponseModel(ProcessingResponseModel):
    """Response schema for STRDS information.
    """
    type = 'object'
    properties = deepcopy(ProcessingResponseModel.properties)
    properties["process_results"] = STRDSInfoModel
    required = deepcopy(ProcessingResponseModel.required)
    # required.append("process_results")
    example = {
        "accept_datetime": "2018-05-02 10:36:43.119861",
        "accept_timestamp": 1525257403.119857,
        "api_info": {
            "endpoint": "strdsmanagementresource",
            "method": "GET",
            "path": "/locations/ECAD/mapsets/PERMANENT/strds/"
                    "precipitation_1950_2013_yearly_mm",
            "request_url": "http://localhost:8080/locations/ECAD/mapsets/"
                           "PERMANENT/strds/precipitation_1950_2013_yearly_mm"
        },
        "datetime": "2018-05-02 10:36:43.677867",
        "http_code": 200,
        "message": "Information gathering for STRDS "
                   "<precipitation_1950_2013_yearly_mm> successful",
        "process_chain_list": [
            {
                "1": {
                    "flags": "g",
                    "inputs": {
                        "input": "precipitation_1950_2013_yearly_mm",
                        "type": "strds"
                    },
                    "module": "t.info"
                }
            }
        ],
        "process_log": [
            {
                "executable": "t.info",
                "parameter": [
                    "type=strds",
                    "input=precipitation_1950_2013_yearly_mm",
                    "-g"
                ],
                "return_code": 0,
                "run_time": 0.4944636821746826,
                "stderr": [
                    ""
                ],
                "stdout": "..."}
        ],
        "process_results": {
            "aggregation_type": "None",
            "semantic_labels": "None",
            "bottom": "0.0",
            "creation_time": "2017-12-29 15:58:40.020820",
            "creator": "soeren",
            "east": "75.5",
            "end_time": "2013-01-01 00:00:00",
            "ewres_max": "0.25",
            "ewres_min": "0.25",
            "granularity": "1 year",
            "id": "precipitation_1950_2013_yearly_mm@PERMANENT",
            "map_time": "interval",
            "mapset": "PERMANENT",
            "max_max": "5132.0",
            "max_min": "2498.3",
            "min_max": "35.2",
            "min_min": "0.0",
            "modification_time": "2017-12-29 15:58:44.396206",
            "name": "precipitation_1950_2013_yearly_mm",
            "north": "75.5",
            "nsres_max": "0.25",
            "nsres_min": "0.25",
            "number_of_semantic_labels": "None",
            "number_of_maps": "63",
            "raster_register": "raster_map_register_87e1edbaf2da4a27a03da04fa9f3a7f1",
            "semantic_type": "mean",
            "south": "25.25",
            "start_time": "1950-01-01 00:00:00",
            "temporal_type": "absolute",
            "top": "0.0",
            "west": "-40.5"
        },
        "progress": {
            "num_of_steps": 1,
            "step": 1
        },
        "resource_id": "resource_id-1f178974-684d-417e-a3f4-878708b7382b",
        "status": "finished",
        "time_delta": 0.5580840110778809,
        "timestamp": 1525257403.6778474,
        "urls": {
            "resources": [],
            "status": "http://localhost:8080/resources/user/"
                      "resource_id-1f178974-684d-417e-a3f4-878708b7382b"
        },
        "user_id": "user"
    }


class STRDSCreationModel(Schema):
    """Schema for STRDS creation
    """
    description = "Information required to create a new STRDS"
    type = 'object'
    properties = {
        'title': {
            'type': 'string',
            'description': 'The title of the STRDS',
        },
        'description': {
            'type': 'string',
            'description': 'The description of the STRDS',
        },
        'temporaltype': {
            'type': 'string',
            'description': 'The temporal type of the STRDS, which can be '
                           'absolute and relative',
            'default': "absolute"
        }
    }
    required = ['title', 'description']
    example = {"title": "Monthly mean temperature from 1950-2010 for Germany",
               "description": "Monthly mean temperature from 1950-2010 for Germany "
                              "in degree celsius",
               "ttype": "absolute"}



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Model for Base layer resources
"""

from flask_restful_swagger_2 import Schema

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Carmen Tawalika"
__copyright__ = "Copyright 2016-2021, Sören Gebbert and mundialis GmbH & Co. KG"
__maintainer__ = "mundialis"


class SetRegionModel(Schema):
    """This schema represents the computational region definition for raster
     and vector map layers.

        n : for north
        s : for south
        e : for east
        w : for west
        res : resolution of the region
        ewres: east-west resolution
        nsres: north-south resolution
        raster: Raster layer name with mapset to set the region from
        align: Raster layer name with mapset to align the region to
        zoom: Raster layer name with mapset to zoom the region to
        vector: Vector layer name with mapset to set the region from

    """
    type = 'object'
    properties = {
        'region': {
            'type': 'string',
            'description': 'Set current region from named region'
        },
        'raster': {
            'type': 'string',
            'description': 'Raster layer name with mapset to set the region from'
        },
        'align': {
            'type': 'string',
            'description': 'Raster layer name with mapset to align the region to'
        },
        'zoom': {
            'type': 'string',
            'description': 'Raster layer name with mapset to zoom the region to'
        },
        'vector': {
            'type': 'string',
            'description': 'Vector layer name with mapset to set the region from'
        },
        'n': {
            'type': 'number',
            'format': 'double',
            'description': 'Value for the northern edge'
        },
        's': {
            'type': 'number',
            'format': 'double',
            'description': 'Value for the southern edge'
        },
        'w': {
            'type': 'number',
            'format': 'double',
            'description': 'Value for the western edge'
        },
        'e': {
            'type': 'number',
            'format': 'double',
            'description': 'Value for the eastern edge'
        },
        't': {
            'type': 'number',
            'format': 'double',
            'description': 'Value for the top edge'
        },
        'b': {
            'type': 'number',
            'format': 'double',
            'description': 'Value for the bottom edge'
        },
        'nsres': {
            'type': 'number',
            'format': 'double',
            'description': 'North-south 2D grid resolution'
        },
        'res': {
            'type': 'number',
            'format': 'double',
            'description': '2D grid resolution (north-south and east-west)'
        },
        'ewres': {
            'type': 'number',
            'format': 'double',
            'description': 'East-west 2D grid resolution'
        },
        'res3': {
            'type': 'number',
            'format': 'double',
            'description': '3D grid resolution (north-south, east-west and top-bottom)'
        },
        'tbres': {
            'type': 'number',
            'format': 'double',
            'description': 'Top-bottom 3D grid resolution'
        }
    }



from pkg_resources import get_distribution, DistributionNotFound

API_VERSION = "v3"

# This is the URL prefix that must be used in the tests
URL_PREFIX = "/api/%s" % API_VERSION

try:
    # Change here if project is renamed and does not equal the package name
    dist_name = __name__
    __version__ = get_distribution(dist_name).version
except DistributionNotFound:
    __version__ = 'unknown'



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
API docs for mapset_management
"""

from actinia_core.models.response_models import (
    MapsetInfoResponseModel,
    ProcessingResponseModel,
    ProcessingErrorResponseModel,
    StringListProcessingResultResponseModel,
)

from actinia_api.swagger2.actinia_core.schemas.mapset_management import (
    MapsetLockManagementResponseModel,
)

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Carmen Tawalika, Anika Weinmann"
__copyright__ = "Copyright 2016-2022, mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"

get_doc = {
    "tags": ["Mapset Management"],
    "description": "Get a list of all mapsets that are located in a "
    "specific location. "
    "Minimum required user role: user.",
    "parameters": [
        {
            "name": "location_name",
            "description": "The name of the location",
            "required": True,
            "in": "path",
            "type": "string",
            "default": "nc_spm_08",
        }
    ],
    "responses": {
        "200": {
            "description": "This response returns a list of mapset names "
            "and the log of the process chain that was used "
            "to create the response.",
            "schema": StringListProcessingResultResponseModel,
        },
        "400": {
            "description": "The error message and a detailed log why listing "
            "of mapsets did not succeeded",
            "schema": ProcessingErrorResponseModel,
        },
    },
}

get_user_doc = {
    "tags": ["Mapset Management"],
    "description": "Get the current computational region of the mapset and the"
    " projection of the location as WKT string. Minimum required "
    "user role: user.",
    "parameters": [
        {
            "name": "location_name",
            "description": "The name of the location",
            "required": True,
            "in": "path",
            "type": "string",
            "default": "nc_spm_08",
        },
        {
            "name": "mapset_name",
            "description": "The name of the mapset",
            "required": True,
            "in": "path",
            "type": "string",
            "default": "PERMANENT",
        },
    ],
    "responses": {
        "200": {
            "description": "The current computational region of the "
            "mapset and the projection of the location",
            "schema": MapsetInfoResponseModel,
        },
        "400": {
            "description": "The error message and a detailed error log",
            "schema": ProcessingErrorResponseModel,
        },
    },
}

post_user_doc = {
    "tags": ["Mapset Management"],
    "description": "Create a new mapset in an existing location. Minimum "
    "required user role: user.",
    "parameters": [
        {
            "name": "location_name",
            "description": "The name of the location",
            "required": True,
            "in": "path",
            "type": "string",
        },
        {
            "name": "mapset_name",
            "description": "The name of the mapset",
            "required": True,
            "in": "path",
            "type": "string",
        },
    ],
    "responses": {
        "200": {
            "description": "Success message for mapset creation",
            "schema": ProcessingResponseModel,
        },
        "400": {
            "description": "The error message and a detailed error log",
            "schema": ProcessingErrorResponseModel,
        },
    },
}

delete_user_doc = {
    "tags": ["Mapset Management"],
    "description": "Delete an existing mapset. Minimum required user role:"
    " user.",
    "parameters": [
        {
            "name": "location_name",
            "description": "The name of the location",
            "required": True,
            "in": "path",
            "type": "string",
        },
        {
            "name": "mapset_name",
            "description": "The name of the mapset",
            "required": True,
            "in": "path",
            "type": "string",
        },
    ],
    "responses": {
        "200": {
            "description": "Success message for mapset deletion",
            "schema": ProcessingResponseModel,
        },
        "400": {
            "description": "The error message and a detailed error log",
            "schema": ProcessingErrorResponseModel,
        },
    },
}

get_lock_doc = {
    "tags": ["Mapset Management"],
    "description": "Get the location/mapset lock status. "
    "Minimum required user role: admin.",
    "parameters": [
        {
            "name": "location_name",
            "description": "The name of the location",
            "required": True,
            "in": "path",
            "type": "string",
            "default": "nc_spm_08",
        },
        {
            "name": "mapset_name",
            "description": "The name of the mapset",
            "required": True,
            "in": "path",
            "type": "string",
            "default": "PERMANENT",
        },
    ],
    "responses": {
        "200": {
            "description": "Get the location/mapset lock status, either "
            '"True" or "None"',
            "schema": MapsetLockManagementResponseModel,
        },
        "400": {
            "description": "The error message and a detailed error log",
            "schema": ProcessingResponseModel,
        },
    },
}

post_lock_doc = {
    "tags": ["Mapset Management"],
    "description": "Create a location/mapset lock. A location/mapset lock can "
    "be created so that no operation can be performed on it "
    "until it is unlocked. "
    "Minimum required user role: admin.",
    "parameters": [
        {
            "name": "location_name",
            "description": "The name of the location",
            "required": True,
            "in": "path",
            "type": "string",
            "default": "nc_spm_08",
        },
        {
            "name": "mapset_name",
            "description": "The name of the mapset",
            "required": True,
            "in": "path",
            "type": "string",
            "default": "PERMANENT",
        },
    ],
    "responses": {
        "200": {
            "description": "Success message if the location/mapset was "
            "locked successfully",
            "schema": ProcessingResponseModel,
        },
        "400": {
            "description": "The error message and a detailed error log",
            "schema": ProcessingResponseModel,
        },
    },
}

delete_lock_doc = {
    "tags": ["Mapset Management"],
    "description": "Delete a location/mapset lock. A location/mapset lock "
    "can be deleted so that operation can be performed on "
    "it until it is locked. "
    "Minimum required user role: admin.",
    "parameters": [
        {
            "name": "location_name",
            "description": "The name of the location",
            "required": True,
            "in": "path",
            "type": "string",
            "default": "nc_spm_08",
        },
        {
            "name": "mapset_name",
            "description": "The name of the mapset",
            "required": True,
            "in": "path",
            "type": "string",
            "default": "PERMANENT",
        },
    ],
    "responses": {
        "200": {
            "description": "Success message if the location/mapset was "
            "unlocked successfully",
            "schema": ProcessingResponseModel,
        },
        "400": {
            "description": "The error message and a detailed error log",
            "schema": ProcessingResponseModel,
        },
    },
}



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2021 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Models for api_log_management
"""

from flask_restful_swagger_2 import Schema

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Carmen Tawalika"
__copyright__ = "Copyright 2016-2021, Sören Gebbert and mundialis GmbH & Co. KG"
__maintainer__ = "mundialis"


class ApiLogEntryModel(Schema):
    """Response schema for a single API log entry that is used to track all API
    calls of a user.
    """
    type = 'object'
    properties = {
        'time_stamp': {
            'type': 'string',
            'description': 'The time stamp of the API call'
        },
        'node': {
            'type': 'string',
            'description': 'The node that executed the API call'
        },
        'endpoint': {
            'type': 'string',
            'description': 'The endpoint of the API call'
        },
        'method': {
            'type': 'string',
            'description': 'The HTTP method of the request'
        },
        'path': {
            'type': 'string',
            'description': 'The path of the REST API call'
        },
        'url': {
            'type': 'string',
            'description': 'The request URL'
        },
        'request_str': {
            'type': 'string',
            'description': 'The request string'
        }
    }
    required = ["time_stamp", "node", "endpoint",
                "method", "path", "url", "request_str"]


class ApiLogListModel(Schema):
    """Response schema that represents a list of API log entries.
    """
    type = 'object'
    properties = {
        'api_log_list': {
            'type': 'array',
            'items': ApiLogEntryModel,
            'description': 'A list of ApiLogEntryModel objects'
        }
    }
    required = ["api_log_list"]



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
API docs for process_validation
"""

from actinia_core.core.common.process_chain import ProcessChainModel
from actinia_core.models.response_models import (
    ProcessingResponseModel,
    ProcessingErrorResponseModel,
)

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Anika Weinmann"
__copyright__ = "Copyright 2016-2022, mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"

DESCR = """Validate a process chain, check the provided sources (links)
and the mapsets. The list of processes that were checked by Actinia are
returned in the JSON response.
"""

post_doc = {
    "tags": ["Processing"],
    "description": DESCR,
    "consumes": ["application/json"],
    "parameters": [
        {
            "name": "location_name",
            "description": "The location name that contains the data that "
            "should be used in the process chain",
            "required": True,
            "in": "path",
            "type": "string",
            "default": "nc_spm_08",
        },
        {
            "name": "process_chain",
            "description": "The process chain that should be validated",
            "required": True,
            "in": "body",
            "schema": ProcessChainModel,
        },
    ],
    "responses": {
        "200": {
            "description": "The result of the process chain validation. "
            "A list of processes that will be executed by "
            "Actinia Core",
            "schema": ProcessingResponseModel,
        },
        "400": {
            "description": "The error message and a detailed log why process "
            "chain validation did not succeeded",
            "schema": ProcessingErrorResponseModel,
        },
    },
}



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
The user specific resources

This module specifies all endpoints to manage user accounts
in the redis database via REST API calls.

TODO: Implement POST full permission creation
      Implement PUT to modify existing users
"""
from flask_restful_swagger_2 import Schema

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Carmen Tawalika"
__copyright__ = "Copyright 2016-2021, Sören Gebbert and mundialis GmbH & Co. KG"
__maintainer__ = "mundialis"


class TokenResponseModel(Schema):
    """Response schema that is used for authentication token generation.

    """
    type = 'object'
    properties = {
        'status': {
            'type': 'string',
            'description': 'The status of the resource, values: success, error'
        },
        'token': {
            'type': 'string',
            'description': 'The generated token for authentication'
        },
        'message': {
            'type': 'string',
            'description': 'The message of the token generation'
        }
    }
    required = ["status", "token"]



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Models for location_management
"""

from flask_restful_swagger_2 import Schema

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Carmen Tawalika"
__copyright__ = "Copyright 2016-2021, Sören Gebbert and mundialis GmbH & Co. KG"
__maintainer__ = "mundialis"


class LocationListResponseModel(Schema):
    """Response schema for location lists
    """
    type = 'object'
    properties = {
        'status': {
            'type': 'string',
            'description': 'The status of the resource, values: accepted, '
                           'running, finished, terminated, error'
        },
        'locations': {
            'type': 'array',
            'items': {"type": "string"},
            'description': 'The list of locations in the GRASS database'
        }
    }
    example = {"locations": ["nc_spm_08", "latlong_wgs84", "ECAD"], "status": "success"}
    required = ["status", "locations"]


class ProjectionInfoModel(Schema):
    """Schema to define projection information as JSON input in POST requests
    """
    type = 'object'
    properties = {
        'epsg': {
            'type': 'string',
            'description': 'The EPSG code of the projection that should be used '
                           'to create a location'
        }
    }
    example = {"epsg": "4326"}
    required = ["epsg"]



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
API docs for ephemeral_processing_with_export
"""

from actinia_core.models.response_models import (
    ProcessingResponseModel,
    ProcessingErrorResponseModel,
)
from actinia_core.core.common.process_chain import ProcessChainModel


__license__ = "GPLv3"
__author__ = "Sören Gebbert, Anika Weinmann"
__copyright__ = "Copyright 2016-2022, mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"

DESCR = """Execute a user defined process chain in an ephemeral database
and provide the generated resources as downloadable files via URL's.
Minimum required user role: user.

The process chain is executed asynchronously. The provided status URL
in the response must be polled to gain information about the processing
progress and finishing status.

**Note**

    Make sure that the process chain definition identifies all raster, vector
    or space-time datasets correctly with name and mapset: name@mapset if you
    use data from other mapsets in the specified location.

    All required mapsets will be identified by analysing the input parameter
    of all module descriptions in the provided process chain and
    mounted read-only into the ephemeral database that is used for processing.

The persistent database will not be modified. The ephemeral database will be
removed after processing.
Use the URL's provided in the finished response to download the resource that
were specified in the process chain for export.

This endpoint also allows the creation of STAC ITEMS through the
ACTINIA STAC PLUGIN. The STAC item is stored in a dedicated
CATALOG following the standard from STAC specification (https://stacspec.org/).
"""

post_doc = {
    "tags": ["Processing"],
    "description": DESCR,
    "consumes": ["application/json"],
    "parameters": [
        {
            "name": "location_name",
            "description": "The location name that contains the data that "
            "should be processed",
            "required": True,
            "in": "path",
            "type": "string",
            "default": "nc_spm_08",
        },
        {
            "name": "process_chain",
            "description": "The process chain that should be executed",
            "required": True,
            "in": "body",
            "schema": ProcessChainModel,
        },
    ],
    "responses": {
        "200": {
            "description": "The result of the process chain execution",
            "schema": ProcessingResponseModel,
        },
        "400": {
            "description": "The error message and a detailed log why process "
            "chain execution did not succeeded",
            "schema": ProcessingErrorResponseModel,
        },
    },
}



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2021-2022 mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
API docs for process_chain_monitoring
"""

from actinia_core.models.response_models import SimpleResponseModel

from actinia_api.swagger2.actinia_core.schemas.process_chain_monitoring \
    import (
        MapsetSizeResponseModel,
        MaxMapsetSizeResponseModel,
    )


__license__ = "GPLv3"
__author__ = "Anika Weinmann, Carmen Tawalika"
__copyright__ = "Copyright 2021-2022, mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"

mapset_size_get_doc = {
    "tags": ["Process Chain Monitoring"],
    "description": "Get the mapset sizes of a resource. "
    "Minimum required user role: user.",
    "parameters": [
        {
            "name": "user_id",
            "description": "The unique user name/id",
            "required": True,
            "in": "path",
            "type": "string",
        },
        {
            "name": "resource_id",
            "description": "The id of the resource",
            "required": True,
            "in": "path",
            "type": "string",
        },
    ],
    "responses": {
        "200": {
            "description": "The current state of the resource",
            "schema": MapsetSizeResponseModel,
        },
        "400": {
            "description": "The error message if the resource does not exist",
            "schema": SimpleResponseModel,
        },
    },
}

mapset_sizediff_get_doc = {
    "tags": ["Process Chain Monitoring"],
    "description": "Get the step-by-step mapset size differences of a "
    "resource. Minimum required user role: user.",
    "parameters": [
        {
            "name": "user_id",
            "description": "The unique user name/id",
            "required": True,
            "in": "path",
            "type": "string",
        },
        {
            "name": "resource_id",
            "description": "The id of the resource",
            "required": True,
            "in": "path",
            "type": "string",
        },
    ],
    "responses": {
        "200": {
            "description": "The current state of the resource",
            "schema": MapsetSizeResponseModel,
        },
        "400": {
            "description": "The error message if the resource does not exists",
            "schema": SimpleResponseModel,
        },
    },
}

mapset_maxsize_get_doc = {
    "tags": ["Process Chain Monitoring"],
    "description": "Get the maximum mapset size of a resource. "
    "Minimum required user role: user.",
    "parameters": [
        {
            "name": "user_id",
            "description": "The unique user name/id",
            "required": True,
            "in": "path",
            "type": "string",
        },
        {
            "name": "resource_id",
            "description": "The id of the resource",
            "required": True,
            "in": "path",
            "type": "string",
        },
    ],
    "responses": {
        "200": {
            "description": "The current state of the resource",
            "schema": MaxMapsetSizeResponseModel,
        },
        "400": {
            "description": "The error message if the resource does not exist",
            "schema": SimpleResponseModel,
        },
    },
}

mapset_render_sizes_get_doc = {
    "tags": ["Process Chain Monitoring"],
    "description": "Render the mapset sizes of a resource. "
    "Minimum required user role: user.",
    "parameters": [
        {
            "name": "user_id",
            "description": "The unique user name/id",
            "required": True,
            "in": "path",
            "type": "string",
        },
        {
            "name": "resource_id",
            "description": "The id of the resource",
            "required": True,
            "in": "path",
            "type": "string",
        },
    ],
    "produces": ["image/png"],
    "responses": {
        "200": {"description": "The PNG image"},
        "400": {
            "description": "The error message and a detailed log why "
            "rendering did not succeed",
            "schema": SimpleResponseModel,
        },
    },
}

mapset_render_sizediff_get_doc = {
    "tags": ["Process Chain Monitoring"],
    "description": "Render the step-by-step mapset size differences of a "
    "resource. Minimum required user role: user.",
    "parameters": [
        {
            "name": "user_id",
            "description": "The unique user name/id",
            "required": True,
            "in": "path",
            "type": "string",
        },
        {
            "name": "resource_id",
            "description": "The id of the resource",
            "required": True,
            "in": "path",
            "type": "string",
        },
    ],
    "produces": ["image/png"],
    "responses": {
        "200": {"description": "The PNG image"},
        "400": {
            "description": "The error message and a detailed log why "
            "rendering did not succeed",
            "schema": SimpleResponseModel,
        },
    },
}



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Vector layer resources
"""
import random
from copy import deepcopy
from flask_restful_swagger_2 import Schema
from actinia_core.models.response_models import \
    ProcessingResponseModel
from actinia_api.swagger2.actinia_core.schemas.map_layer_base import SetRegionModel

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Carmen Tawalika"
__copyright__ = "Copyright 2016-2021, Sören Gebbert and mundialis GmbH & Co. KG"
__maintainer__ = "mundialis"


class VectorAttributeModel(Schema):
    """Simple model that represent the description of vector attributes
    """
    description = "Description of a vector map layer attribute"
    type = 'object'
    properties = {'column': {'type': 'string'},
                  'type': {'type': 'string'}}
    example = {"cat": "INTEGER", "z": "DOUBLE PRECISION"}


class VectorInfoModel(Schema):
    """Schema that contains vector map layer information
    """
    description = "Description of a GRASS GIS vector map layer"
    type = 'object'
    properties = {
        'Attributes': {'type': 'array', "items": VectorAttributeModel},
        'COMMAND': {'type': 'string'},
        'areas': {'type': 'string'},
        'bottom': {'type': 'string'},
        'boundaries': {'type': 'string'},
        'centroids': {'type': 'string'},
        'comment': {'type': 'string'},
        'creator': {'type': 'string'},
        'database': {'type': 'string'},
        'digitization_threshold': {'type': 'string'},
        'east': {'type': 'string'},
        'faces': {'type': 'string'},
        'format': {'type': 'string'},
        'holes': {'type': 'string'},
        'islands': {'type': 'string'},
        'kernels': {'type': 'string'},
        'level': {'type': 'string'},
        'lines': {'type': 'string'},
        'location': {'type': 'string'},
        'map3d': {'type': 'string'},
        'mapset': {'type': 'string'},
        'name': {'type': 'string'},
        'nodes': {'type': 'string'},
        'north': {'type': 'string'},
        'num_dblinks': {'type': 'string'},
        'organization': {'type': 'string'},
        'points': {'type': 'string'},
        'primitives': {'type': 'string'},
        'projection': {'type': 'string'},
        'zone': {'type': 'string'},
        'scale': {'type': 'string'},
        'source_date': {'type': 'string'},
        'south': {'type': 'string'},
        'timestamp': {'type': 'string'},
        'title': {'type': 'string'},
        'top': {'type': 'string'},
        'volumes': {'type': 'string'},
        'west': {'type': 'string'},
        'attribute_layer_name': {'type': 'string'},
        'attribute_table': {'type': 'string'},
        'attribute_database_driver': {'type': 'string'},
        'attribute_database': {'type': 'string'},
        'attribute_primary_key': {'type': 'string'},
        'attribute_layer_number': {'type': 'string'},
    }
    example = {
        "Attributes": [
            {"column": "cat", "type": "INTEGER"},
            {"column": "z", "type": "DOUBLE PRECISION"}
        ],
        "COMMAND": " v.random -z output=\"test_layer\" npoints=1 layer=\"-1\" "
                   "zmin=1.0 zmax=1.0 seed=1 column=\"z\" "
                   "column_type=\"double precision\"",
        "areas": "0",
        "bottom": "1.000000",
        "boundaries": "0",
        "centroids": "0",
        "comment": "",
        "creator": "soeren",
        "database": "/tmp/gisdbase_b83bebdb543440c7b9991e2e5602ba91",
        "digitization_threshold": "0.000000",
        "east": "644375.544828422",
        "faces": "0",
        "format": "native",
        "holes": "0",
        "islands": "0",
        "kernels": "0",
        "level": "2",
        "lines": "0",
        "location": "nc_spm_08",
        "map3d": "1",
        "mapset": "user1",
        "name": "test_layer",
        "nodes": "0",
        "north": "221135.648003836",
        "num_dblinks": "1",
        "organization": "",
        "points": "1",
        "primitives": "1",
        "projection": "Lambert Conformal Conic",
        "scale": "1:1",
        "source_date": "Thu May 18 21:40:02 2017",
        "south": "221135.648003836",
        "timestamp": "none",
        "title": "",
        "top": "1.000000",
        "volumes": "0",
        "west": "644375.544828422",
        "attribute_database": "/tmp/gisdbase_eabed7327ec84d219698670884136c2a/"
                              "nc_spm_08/user1/vector/test_layer/sqlite.db",
        "attribute_database_driver": "sqlite",
        "attribute_layer_name": "test_layer",
        "attribute_layer_number": "1",
        "attribute_primary_key": "cat",
        "attribute_table": "test_layer",
    }


class VectorInfoResponseModel(ProcessingResponseModel):
    """Response schema for vector map layer information.
    """
    type = 'object'
    properties = deepcopy(ProcessingResponseModel.properties)
    properties["process_results"] = VectorInfoModel
    required = deepcopy(ProcessingResponseModel.required)
    # required.append("process_results")
    example = {
        "accept_datetime": "2018-05-06 21:36:53.825043",
        "accept_timestamp": 1525635413.8250418,
        "api_info": {
            "endpoint": "vectorlayerresource",
            "method": "GET",
            "path": "/locations/nc_spm_08/mapsets/PERMANENT/vector_layers/geology",
            "request_url": "http://localhost:5000/locations/nc_spm_08/mapsets/"
                           "PERMANENT/vector_layers/geology"
        },
        "datetime": "2018-05-06 21:36:54.032325",
        "http_code": 200,
        "message": "Processing successfully finished",
        "process_chain_list": [
            {
                "1": {
                    "flags": "gte",
                    "inputs": {
                        "map": "geology@PERMANENT"
                    },
                    "module": "v.info"
                },
                "2": {
                    "flags": "h",
                    "inputs": {
                        "map": "geology@PERMANENT"
                    },
                    "module": "v.info"
                },
                "3": {
                    "flags": "c",
                    "inputs": {
                        "map": "geology@PERMANENT"
                    },
                    "module": "v.info"
                }
            }
        ],
        "process_log": [
            {
                "executable": "v.info",
                "parameter": [
                    "map=geology@PERMANENT",
                    "-gte"
                ],
                "return_code": 0,
                "run_time": 0.050188302993774414,
                "stderr": [
                    ""
                ],
                "stdout": "..."
            },
            {
                "executable": "v.info",
                "parameter": [
                    "map=geology@PERMANENT",
                    "-h"
                ],
                "return_code": 0,
                "run_time": 0.05018758773803711,
                "stderr": [
                    ""
                ],
                "stdout": "..."
            },
            {
                "executable": "v.info",
                "parameter": [
                    "map=geology@PERMANENT",
                    "-c"
                ],
                "return_code": 0,
                "run_time": 0.050185441970825195,
                "stderr": [
                    "Displaying column types/names for database connection of "
                    "layer <1>:",
                    ""
                ],
                "stdout": "..."
            }
        ],
        "process_results": {
            "Attributes": [
                {
                    "column": "cat",
                    "type": "INTEGER"
                },
                {
                    "column": "onemap_pro",
                    "type": "DOUBLE PRECISION"
                },
                {
                    "column": "PERIMETER",
                    "type": "DOUBLE PRECISION"
                },
                {
                    "column": "GEOL250_",
                    "type": "INTEGER"
                },
                {
                    "column": "GEOL250_ID",
                    "type": "INTEGER"
                },
                {
                    "column": "GEO_NAME",
                    "type": "CHARACTER"
                },
                {
                    "column": "SHAPE_area",
                    "type": "DOUBLE PRECISION"
                },
                {
                    "column": "SHAPE_len",
                    "type": "DOUBLE PRECISION"
                }
            ],
            "COMMAND": " v.db.connect -o map=\"geology@PERMANENT\" "
                       "driver=\"sqlite\" database=\"$GISDBASE/$LOCATION_NAME/"
                       "$MAPSET/sqlite/sqlite.db\" table=\"geology\" key=\"cat\" "
                       "layer=\"1\" separator=\"|\"",
            "areas": "1832",
            "attribute_database": "/home/soeren/actinia/workspace/temp_db/"
                                  "gisdbase_d98fc0548fc44fac8fe43abd575e98cc/"
                                  "nc_spm_08/PERMANENT/sqlite/sqlite.db",
            "attribute_database_driver": "sqlite",
            "attribute_layer_name": "geology",
            "attribute_layer_number": "1",
            "attribute_primary_key": "cat",
            "attribute_table": "geology",
            "bottom": "0.000000",
            "boundaries": "3649",
            "centroids": "1832",
            "comment": "",
            "creator": "helena",
            "database": "/home/soeren/actinia/workspace/temp_db/"
                        "gisdbase_d98fc0548fc44fac8fe43abd575e98cc",
            "digitization_threshold": "0.000000",
            "east": "930172.312822711",
            "format": "native",
            "islands": "907",
            "level": "2",
            "lines": "0",
            "location": "nc_spm_08",
            "map3d": "0",
            "mapset": "PERMANENT",
            "name": "geology",
            "nodes": "2724",
            "north": "318117.437416345",
            "num_dblinks": "1",
            "organization": "NC OneMap",
            "points": "0",
            "primitives": "5481",
            "projection": "Lambert Conformal Conic",
            "scale": "1:1",
            "source_date": "Mon Nov  6 15:48:53 2006",
            "south": "10875.8272320917",
            "timestamp": "none",
            "title": "North Carolina geology map (polygon map)",
            "top": "0.000000",
            "west": "123971.194989783"
        },
        "progress": {
            "num_of_steps": 3,
            "step": 3
        },
        "resource_id": "resource_id-5494af8c-8c9d-4f8e-a568-d6e86d69d8fd",
        "status": "finished",
        "time_delta": 0.20732927322387695,
        "timestamp": 1525635414.0323067,
        "urls": {
            "resources": [],
            "status": "http://localhost:5000/resources/user/"
                      "resource_id-5494af8c-8c9d-4f8e-a568-d6e86d69d8fd"
        },
        "user_id": "user"
    }


class VectorCreationModel(Schema):
    """Schema for input parameters to generate a random point vector map layer
    """
    type = 'object'
    properties = {
        'npoints': {
            'type': 'number',
            'format': 'integer',
            'description': 'Number of points to be created',
            'default': 5
        },
        'seed': {
            'type': 'number',
            'format': 'integer',
            'description': 'The seed to initialize the random generator. '
                           'If not set the process ID is used',
            'default': random.randint(0, 1000000)
        },
        'zmin': {
            'type': 'number',
            'format': 'double',
            'description': 'Minimum z height',
            'default': 0.0
        },
        'zmax': {
            'type': 'number',
            'format': 'double',
            'description': 'Maximum z height',
            'default': 100.0
        }
    }


class VectorRegionCreationModel(Schema):
    """Schema for random vector generation in a specific region
    """
    type = 'object'
    properties = {
        'region': SetRegionModel,
        'parameter': VectorCreationModel
    }
    example = {"region": {"n": 228500, "s": 215000,
                          "e": 645000, "w": 630000},
               "parameter": {"npoints": 1, "zmin": 1,
                             "zmax": 1, "seed": 1}}



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
API docs for download_cache_management
"""

from actinia_core.models.response_models import (
    StorageResponseModel,
    ProcessingResponseModel,
)

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Anika Weinmann"
__copyright__ = "Copyright 2016-2022, mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"


get_doc = {
    "tags": ["Cache Management"],
    "description": "Get the current size of the download cache. "
    "Minimum required user role: admin.",
    "responses": {
        "200": {
            "description": "The current state of the download cache",
            "schema": StorageResponseModel,
        },
        "400": {
            "description": "The error message why cache information "
            "gathering did not succeeded",
            "schema": ProcessingResponseModel,
        },
    },
}


delete_doc = {
    "tags": ["Cache Management"],
    "description": "Clean the download cache and remove all cached data. "
    "Minimum required user role: admin.",
    "responses": {
        "200": {
            "description": "Processing status of cache deletion",
            "schema": StorageResponseModel,
        },
        "400": {
            "description": "The error message why cache cleaning did not succeeded",
            "schema": ProcessingResponseModel,
        },
    },
}
